#ifndef VERTMAT_MATERIAL_H
#define VERTMAT_MATERIAL_H

#include <GL/gl.h>
#include <nmath/vec3.h>
#include <nmath/vec4.h>

#include <boost/shared_ptr.hpp>

using namespace nacb;

class Material {
 public:
  Material() {
    color = Vec4f(0.8, 0.8, 0.8);
    tex = 0;
  }
  
  ~Material() {
    glDeleteTextures(1, &tex);
  }

  void loadTexture(nacb::Image8 & image) {
    if (!tex) {
      glGenTextures(1, &tex);
    }
    glBindTexture(GL_TEXTURE_2D, tex);
    image.initTexture();
    glBindTexture(GL_TEXTURE_2D, 0);
  }

  void bind() {
    if (tex) { 
      glEnable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D, tex);
    }
    else {
      glDisable(GL_TEXTURE_2D);
    }      
    glColor4fv(color.data);
  }

  void unbind() {
    if (tex) { 
      glDisable(GL_TEXTURE_2D);
      glBindTexture(GL_TEXTURE_2D, tex);
    }
  }
  
  static boost::shared_ptr<Material> basicWithTexture(const char * fname) {
    boost::shared_ptr<Material> mat(new Material());
    nacb::Image8 image;
    image.load(fname);
    mat->loadTexture(image);
    return mat;
  }


  Vec4f color;
  GLuint tex;

 public:
  typedef boost::shared_ptr<Material> ptr;
};

#endif // VERTMAT_MATERIAL_H
