#ifndef VIRTMAT_SCENE_H
#define VIRTMAT_SCENE_H

#include <GL/glew.h>

#include "Spring.h"
#include "TetMesh.h"
#include "TetBoundMesh.h"
#include "Globals.h"

#include <utigl/fbo.h>
#include <utigl/glwindow.h>
#include <nmisc/timer.h>

#include <fstream>

#include <boost/shared_ptr.hpp>


#include "SceneDescription.h"

/**
   Simple scene to hold the tetmesh (and some springs)
   and to abstract some of the interaction from a particular UI.
 */
class Scene {
public:
  Scene(const char * descName);
  Scene(const char * tetName,
	const char * tetBound);
  void init();
  
  void draw();
  void drawHud();
  /**
     Super quick hack for drawing a projected ground shadow.
  */
  void drawGround();

  void simulate(double dt = 0.01);
  void reset();

  /**
     Make the current mouse handle a permanent spring.
  */
  void convertHandleToSpring();

  bool handleKeyboard(unsigned char c, int x, int y);
  void moveHandleForView(InteractiveCameraView * view, int x, int y);
  void selectHandleForView(InteractiveCameraView * view, int button, int down, int x, int y);
  void removeSpringForView(InteractiveCameraView * view, int button, int down, int x, int y);

public:
  TetMesh::ptr tetmesh;
  TetBoundMesh boundMesh;

  bool drawTets;
  bool simulating;

  int    handleTet;
  Vec4f handleBary;
  Vec3f handlePos;

  bool wireframe;
  double fps;

  FrameBufferObject * fbo;

  std::vector<TetMeshFractureInfo> recordedFractures;
  std::vector<Spring> springs;

  InteractiveCameraView * view;

  typedef boost::shared_ptr<Scene> ptr;
};


#endif //VIRTMAT_SCENE_H

