#include <string>
#include <vector>

#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/set.hpp>

#include <nmath/vec3.h>
#include <nmath/quaternion.h>

#include "TetMesh.h"

#include <fstream>

#include "Spring.h"
#include "Globals.h"

using namespace nacb;
using namespace std;



class SceneDescription {
 public:

  class TetMesh {
  public:
    TetMesh() { 
      fracThresh = 1000;
    }

    template<class Archive>
      void serialize(Archive & ar, const unsigned int version) {
      ar & fileName & mat.e & mat.v & fracThresh & mat.c_yield & mat.c_max & mat.c_creep;
    }
    
  public:    
    std::string fileName;
    ::Tetrahedron::Material mat;
    double fracThresh;
  };

  class Material {
  public:
    Material() { }

    template<class Archive>
      void serialize(Archive & ar, const unsigned int version) {
      ar & shader & textures;
    }

  public:
    std::string shader;
    std::vector<std::string> textures;
  };

  class TetBoundMesh {
  public:
    TetBoundMesh() {} 

    template<class Archive>
      void serialize(Archive & ar, const unsigned int version) {
      ar & fileName & material;
    }
  public:
    std::string fileName;
    Material material;
  };

  class BasicObject {
  public:
    BasicObject() { 
      pos = Vec3f(0, 0, 0);
      rot = Quaternion();
    }

    template<class Archive>
      void serialize(Archive & ar, const unsigned int version) {
      ar & fileName & pos.x & pos.y & pos.z 
	 & rot.v.x & rot.v.y & rot.v.z & rot.a 
	 & material;
    }

  public:
    std::string fileName;    
    Vec3f pos;
    Quaternion rot;
    Material material;
  };

  SceneDescription() { }

  template<class Archive>
    void serialize(Archive & ar, const unsigned int version) {
    ar & drawTets & tetMesh & tetBoundMesh & basicObjects & springs;
  }

 public:
  bool drawTets;
  TetMesh tetMesh;
  TetBoundMesh tetBoundMesh;
  vector<BasicObject> basicObjects;
  vector<Spring> springs;
};

