#ifndef SPRING_H
#define SPRING_H

#include <GL/glew.h>
#include <nmath/vec3.h>
#include <nmath/vec4.h>
#include <GL/gl.h>

#include <ik/mesh.h>

using namespace nacb;

/**
   This isn't really a spring, rather more of a
   constraint.  It ends up acting like a spring, however.
   The spring is bound to the world coordinate system and
   also a tetrahedron.
*/
class Spring {
public:
  Spring(int _tet = 0, 
	 const Vec4f & _bary = Vec4f(0.25, 0.25, 0.25, 0.25),  
	 const Vec3f & _pos = Vec3f(0, 0, 0)) : 
    tet(_tet), bary(_bary), pos(_pos) 
  { 
    makeGeom();
  }

  void makeGeom() {
    if (!s_baseGeom.vert.size()) {
      s_baseGeom.loadobj("data/spring_base.obj");
      s_springGeom.loadobj("data/spring_middle.obj");
      printf("Made geom: %d, %d\n", s_baseGeom.vert.size());
    }
  }

  void draw(const Vec3f & worldPoint) {
    glPushAttrib(GL_ALL_ATTRIB_BITS);

    if (s_springGeom.vert.size()) {
      glPushMatrix();
      glTranslatef(pos.x, pos.y, pos.z);
      s_baseGeom.draw();
      glPopMatrix();

      glPushMatrix();
      glTranslatef(worldPoint.x, worldPoint.y, worldPoint.z);
      s_baseGeom.draw();
      glPopMatrix();

      Vec3f mid = (pos + worldPoint)*0.5;
      Vec3f diff = (worldPoint - pos);
      double dlen = diff.normalize();
      if (dlen > 0.001) {
	double ry = acos(diff.z);
	double rz = atan2(diff.y, diff.x);
	glPushMatrix();
	glTranslatef(pos.x, pos.y, pos.z);//mid.x, mid.y, mid.z);
	
	glRotatef(rz*180/M_PI, 0, 0, 1);
	glRotatef(ry*180/M_PI, 0, 1, 0);

	glScalef(1, 1, dlen*0.5);
	s_springGeom.draw();

	glPopMatrix();
      }
    }
    else {

      glBegin(GL_LINES);
       glVertex3fv(worldPoint.data);
       glVertex3fv(pos.data);
      glEnd();
    }
  }

  template<class Archive>
    void serialize(Archive & ar, const unsigned int version) {
    ar & tet & bary.x & bary.y & bary.z & pos.x & pos.y & pos.z;
  }

  static Mesh s_baseGeom;
  static Mesh s_springGeom;
  
public:  
  int    tet;
  Vec4f bary;
  Vec3f pos;
};


#endif
