/**
   This is the collosal mess of code that I started writing to handle
   the fracture of the mesh.  The principle is the same, but it didn't categorize
   each triangle directly, merely tried to handle some common cases.

   This did end up working (sometimes), but the code had to be cleaned up.

   There are mappings to added points and to vertices (and from edges) to these
   things.  The code would have been pretty hard to decipher in a little while.

   I believe there was some problems linking face points through the fractured
   triangles (within a fracture).  The second implementation, which treated
   vertices that intersecting vertices and edges directly seemed better.
   
   Furthermore, this implementation did the collision in world space.  I didn't
   realize that this causes problems when th tets are in a strange dangling 
   configuration (e.g., when there is no collisoin detection, the tetrahedra
   can dangle over a separating triangle, causing splits where they shouldn't be.
   That's not to say that performing the collisoin in rest space doesn't come with
   any problems, because it does.
*/

#include "TetBoundMesh.h"

/**
   Keep track of the points that were added during the split.
   Useful for linking the polygon.
*/
struct AddedPoint {
  int point;
  int  face;
  int v1, v2;
  bool  edge;
  bool hasBottomPoint;
  set<int> adjacent;
    
  AddedPoint (int _p, int _f, int _v1, int _v2, bool _edge = true) : 
    point(_p), face(_f), v1(_v1), v2(_v2), edge(_edge) {      
    hasBottomPoint = edge; // All edges have bottom points
  }

  void printAdj() {
    std::set<int>::iterator it(adjacent.begin());
    std::set<int>::iterator last(adjacent.end());
      
    while (it != last) {
      printf(" %d, ",  *it);
      ++it;
    }
    printf("\n");
  }

  int bottomPoint() {
    if (hasBottomPoint) return point + 1;
    else return point;
  }
};


bool TetBoundMesh::fracture2() {
   
  for (int fraci=0; fraci<tets->fractures.size(); fraci++) {
    /*
      For each triangle in the separatingTriangles list,
      we need to find all triangles with edges that intersect it.
      Also need to find all triangles that intersect an edge separating triangle.
      (There should be pairs of these).	
	
      For each new vertex, need to find a path that links them.
      These paths will be linked through the tetrahedra vertices.
      Need to start at some path edge (or the end), and go through
      the separating triangle edges until other path starts are found,
      or until sep-triangle vertics are found.  The closed path should
      be triangulated.	 
    */
    TetMeshFractureInfo & frac = tets->fractures[fraci];     
    std::map<int, bool> global_face_point_above;

    for (int sepi=0; sepi<frac.sepTris.size(); sepi++) {
      TetMeshFractureInfo::SeparatingTriangle sep = frac.sepTris[sepi];
      TriangleIndices triInds = 
	Tetrahedron::tris[sep.tetAboveTriIndex].mapIndices(tets->tets[sep.tetAbove].vi);
      Triangle triangle(tets->vert, triInds);

      if (triangle.below(tets->getWorldCoordinates(sep.tetAbove, Vec4f(0.5, 0.5, 0.5, 0.5)))) {
	printf("Tet above is on the wrong side!\n");
      }

      std::map<int, int> edge_to_point;	
      std::vector<AddedPoint> added_points;
      std::map<int, int> vert_to_added;

      vector<int> sep_edge_points[3];

      isect.push_back(triangle.vert[0]);
      isect.push_back(triangle.vert[1]);
      isect.push_back(triangle.vert[2]);

      // Use a stride bigger than the number vert, as we will be adding some.
      int stride = vert.size() * 4 + 100;
      int ntris = tris.size(); // May add triangles that don't need testing
	
      // Collect all the intersection points on the triangles.	
      for (int trii=0; trii<ntris; trii++) {
	vector<int> isected_edges;
	vector<int> added_by_this_face;

	for (int h=0; h<3; h++){
	  int vi1, vi2;
	  Vec3f v1 = vert[vi1 = tris[trii].vi[h]];
	  Vec3f v2 = vert[vi2 = tris[trii].vi[(h + 1) % 3]];
	  double t = 0;

	  // Check to see if a point was already added on this edge.
	  int index1 = vi1 * stride + vi2;
	  int index2 = vi2 * stride + vi1;

	  if (edge_to_point.count(index1) ||
	      edge_to_point.count(index2)) {
	    isected_edges.push_back(h);
	    added_by_this_face.push_back(vert_to_added[edge_to_point[index1]]);
	    continue;
	  }

	  if (triangle.intersectSegment(v1, v2, t)) {
	    if (t <= 1e-5 || t>= 1.0 - 1e-5) {
	      printf("Warning: t close to point; %f\n", t);
	    }
	    double d1, d2;
	    if ((d1 = triangle.distance(v1)) < 1e-5 ||
		(d2 = triangle.distance(v2)) < 1e-5) {
	      printf("Warning: a point is close to the plane %f %f\n", d1, d2);
	      continue;
	    }

	    isect.push_back(v2*t + v1*(1.0 - t));
	    isected_edges.push_back(h);

	    edge_to_point[index1] = vert.size();
	    edge_to_point[index2] = vert.size();

	    vert_to_added[vert.size()] = added_points.size();
	    vert_to_added[vert.size()+1] = added_points.size();
	    added_by_this_face.push_back(added_points.size());
	    added_points.push_back(AddedPoint(vert.size(), trii, vi1, vi2));
	      

	    // Add two extra points (after each other; one above plane, one below).
	    Vec3f newVert, newRestVert;
	    newRestVert = restVert[vi1]*(1.0 - t) + restVert[vi2]*t;
	    newVert = vert[vi1]*(1.0 - t) + vert[vi2]*t;
	      
	    binding.push_back(TetBinding(sep.tetAbove,
					 tets->getBaryCoordinates(sep.tetAbove, 
								  newVert)));
	    binding.push_back(TetBinding(sep.tetBelow,
					 tets->getBaryCoordinates(sep.tetBelow, 
								  newVert)));

	    for (int i=0; i<2; i++) {
	      restVert.push_back(newRestVert);
	      vert.push_back(newVert);
	    }
	  }
	}

	// Fix up the adjacency for the points.
	for (int i=0; i<added_by_this_face.size(); i++) {
	  for (int j=i+1; j<added_by_this_face.size(); j++) {
	    added_points[added_by_this_face[i]].
	      adjacent.insert(added_by_this_face[j]);
	    added_points[added_by_this_face[j]].
	      adjacent.insert(added_by_this_face[i]);
	  }	
	}
	  
	if (isected_edges.size()) {
	  // Check for sep triangle edges intersecting
	  Triangle triangle2(vert, tris[trii].vi);
	    
	  vector<Vec3f> facePoints;
	  vector<bool> facePointSplitting;
	  vector<int> facePointEdge;

	  for (int h=0; h<3; h++) {
	    Vec3f v1 = triangle.vert[h];
	    Vec3f v2 = triangle.vert[(h + 1) % 3];
	    double t = 0;

	    if (triangle2.intersectSegment(v1, v2, t)) {
	      // Edge is a splitting edge if it is attached to the split vertex.
	      double thresh = 1e-6;
	      if (triangle.distance(vert[tris[trii].vi[0]]) < thresh ||
		  triangle.distance(vert[tris[trii].vi[1]]) < thresh ||
		  triangle.distance(vert[tris[trii].vi[2]]) < thresh)
		continue;
		    
	      bool splitting = (triInds[h] == frac.vold) ||
		(triInds[(h + 1) % 3] == frac.vold);
	      facePoints.push_back(v2*t + v1*(1.0 - t));
	      facePointSplitting.push_back(splitting);
	      facePointEdge.push_back(h);
	    }
	  }
	  if (facePoints.size())
	    printf("Found sep edge stabbing face (%d) %f %f %f\n", 
		   facePoints.size(), 
		   triangle.distance(vert[tris[trii].vi[0]]),
		   triangle.distance(vert[tris[trii].vi[1]]),
		   triangle.distance(vert[tris[trii].vi[2]]));
							       

	  for (int k=0; k<facePoints.size(); k++) {
	    isect.push_back(facePoints[k]);
	  }	    

	  // We can always create the split triangles.
	  if (isected_edges.size() == 2 && !facePoints.size()) {
	    // e0, e1, and either one or two verts make up the triangle.
	    int vi[3] = {tris[trii].vi[0],
			 tris[trii].vi[1],
			 tris[trii].vi[2]};
	    int e[2] = {edge_to_point[vi[isected_edges[0]]*stride + 
				      vi[(isected_edges[0] + 1)%3]],
			edge_to_point[vi[isected_edges[1]]*stride + 
				      vi[(isected_edges[1] + 1)%3]]};
	    // FIXME: normals and texture coordinates...
	    // The first vertex in the edge pair is on the top (>= 0)
	    int downs[3] = {triangle.above(vert[tris[trii].vi[0]]),
			    triangle.above(vert[tris[trii].vi[1]]),
			    triangle.above(vert[tris[trii].vi[2]])};
	    int down = downs[isected_edges[0]];
		     
	    tris[trii].vi[0] = vi[isected_edges[0]];
	    tris[trii].vi[1] = e[0] + down;
	    tris[trii].vi[2] = e[1] + down;

	    // Fix up adjacency (edges can only go in one direction)
	    {
	      int a0 = vert_to_added[e[0]];
	      int a1 = vert_to_added[e[1]];
	      if (down)
		added_points[a1].adjacent.erase(a0);
	      else
		added_points[a0].adjacent.erase(a1);
	    }

	    if (isected_edges[1] == (isected_edges[0] + 1)) {
	      if (downs[isected_edges[0]] != downs[(isected_edges[0] + 2) %3]){
		printf("Down conflict 0\n");
	      }
	      if (downs[isected_edges[0]] == downs[(isected_edges[0] + 1) %3]){
		printf("Down conflict 1\n");
	      }

	      // The first triangle is on the same side as above
	      tris.push_back(Mesh::Tri());
	      tris.back().vi[0] = vi[isected_edges[0]];
	      tris.back().vi[1] = e[1] + down;
	      tris.back().vi[2] = vi[(isected_edges[0] + 2) % 3];

	      // THis one is on the opposite side.
	      tris.push_back(Mesh::Tri());
	      tris.back().vi[0] = e[0] + !down;
	      tris.back().vi[1] = vi[(isected_edges[0] + 1) % 3];
	      tris.back().vi[2] = e[1] + !down;
	    }
	    else {
	      if (downs[(isected_edges[0] + 1) %3 ] == downs[isected_edges[0]]){
		printf("Down conflict 2\n");
	      }
	      if (downs[(isected_edges[0] + 2) %3 ] == downs[isected_edges[0]]){
		printf("Down conflict 3\n");
	      }
	      if (downs[(isected_edges[0] + 2) %3 ] != downs[(isected_edges[0] + 1) % 3]){
		printf("Down conflict 4\n");
	      }

	      // The other triangles are on the other side.
	      tris.push_back(Mesh::Tri());		
	      tris.back().vi[0] = e[0] + !down;
	      tris.back().vi[1] = vi[(isected_edges[0] + 1)%3];
	      tris.back().vi[2] = vi[(isected_edges[0] + 2)%3];
		
	      tris.push_back(Mesh::Tri());
	      tris.back().vi[0] = e[0] + !down;
	      tris.back().vi[1] = vi[(isected_edges[0] + 2)%3];
	      tris.back().vi[2] = e[1] + !down;
	    }
	  }
	  else if (facePoints.size() == 1) {
	    // What to do here?
	    printf("Face: %d, %d\n", isected_edges.size(), facePoints.size());
	    // Keep track of the top and bottom points so that we can
	    // use them later (in the next separating triangle that shares the
	    // edge that caused this facePoint.
	    TriangleIndices vi(tris[trii].vi);
	    int e = edge_to_point[vi[isected_edges[0]]*stride + 
				  vi[(isected_edges[0] + 1)%3]];
	    int downs[3] = {triangle.above(vert[vi[isected_edges[0]]]),
			    triangle.above(vert[vi[(isected_edges[0] + 1) % 3]]),
			    triangle.above(vert[vi[(isected_edges[0] + 2) % 3]])};
			      
	    int down = downs[0];
	    {
	      bool hasDown = false;
	      int c =vert.size();
	      Vec4f bary = tets->getBaryCoordinates(sep.tetAbove, facePoints[0]);
		
	      printf("face added point: %d (%d)\n", added_points.size(), vert.size());
	      int added = added_points.size();

	      sep_edge_points[facePointEdge[0]].push_back(vert.size());
	      vert_to_added[vert.size()] = added_points.size();
	      added_by_this_face.push_back(added_points.size());
	      added_points.push_back(AddedPoint(vert.size(), trii, -1, -1, false));
	      binding.push_back(TetBinding(sep.tetAbove, bary));
	      restVert.push_back(tets->tets[sep.tetAbove].getWorldCoordinates(tets->restVert,
									      bary));
	      global_face_point_above[vert.size()] = true;
	      vert.push_back(facePoints[0]);
		

	      // If this is a splitting edge, need to add the point 
	      // (actually need to cache this).
	      if (facePointSplitting[0]) {
		bary = tets->getBaryCoordinates(sep.tetBelow, facePoints[0]);
		hasDown = true;
		vert_to_added[vert.size()] = int(added_points.size()) - 1;
		added_points.back().hasBottomPoint = true;

		binding.push_back(TetBinding(sep.tetBelow, bary));
		restVert.push_back(tets->tets[sep.tetBelow].getWorldCoordinates(tets->restVert,
										bary));
		global_face_point_above[vert.size()] = false;
		vert.push_back(facePoints[0]);
	      }

	      // FIXME: this could all be done in one pass at the end.
	      // Fix up the adjacency for the points.
	      /*
		for (int i=0; i<added_by_this_face.size(); i++) {
		for (int j=i+1; j<added_by_this_face.size(); j++) {
		added_points[added_by_this_face[i]].
		adjacent.insert(added_by_this_face[j]);
		added_points[added_by_this_face[j]].
		adjacent.insert(added_by_this_face[i]);
		}	
		}
	      */
	      // Update the triangles.
	      // v1-e-c, e'-v2-c', (down determined by the pair) v2-v3-c, v3-v1-c
	      // in the last two disagreeing pairs will be by default connected to the top.		
	      tris[trii].vi[0] = vi[isected_edges[0]];
	      tris[trii].vi[1] = e + down;
	      tris[trii].vi[2] = c + hasDown * down;

	      if (down) 
		added_points[vert_to_added[e]].adjacent.insert(added);
	      else 
		added_points[added].adjacent.insert(vert_to_added[e]);
		
	      tris.push_back(Mesh::Tri());
	      tris.back().vi[0] = e + !down;
	      tris.back().vi[1] = vi[(isected_edges[0] + 1) % 3];
	      tris.back().vi[2] = c + hasDown * (!down);
	
	      // Use the connected vertices to make a best guess of which
	      // point (top or bottom) these triangles should be connected to.
	      tris.push_back(Mesh::Tri());
	      tris.back().vi[0] = vi[(isected_edges[0] + 1) % 3];
	      tris.back().vi[1] = vi[(isected_edges[0] + 2) % 3];
	      tris.back().vi[2] = c + hasDown * (downs[1] && downs[2]);

	      // Swapping the triangles ensures the triangle that needs to 
	      // be split will be considered.
	      if (hasDown && downs[1] != downs[2]) {
		swap(tris.back(), tris[trii]);
	      }

	      // Swapping the triangles ensures the triangle that needs to 
	      // be split will be considered.
	      tris.push_back(Mesh::Tri());
	      tris.back().vi[0] = vi[(isected_edges[0] + 2) % 3];
	      tris.back().vi[1] = vi[isected_edges[0]];
	      tris.back().vi[2] = c + hasDown * (downs[0] && downs[2]);

	      if (hasDown && downs[0] != downs[2]) {
		swap(tris.back(), tris[trii]);
	      }
	      printf("trii: %d\n", trii);
	    }
	  }
	  else {
	    // Even worse, what to do here.
	    fprintf(stderr, "OMG: facePoints.size() = %d, edges: %d (trii=%d)\n", 
		    facePoints.size(), isected_edges.size(), trii);

	    for (int k=0; k<3; k++) {
	      int vi = tris[trii].vi[k];
	      double diff = triangle.distance(vert[vi]);
	      int added = vert_to_added.count(vi) ? 
		vert_to_added[vi] : -1;

	      if (global_face_point_above.count(vi) ){
		if (added < 0 && diff < 1e-6) {
		  int vup = vi - !global_face_point_above[vi];
		  int edge = 0; // Edge 
		  double edgeDistance = 1e10;
		  for (int e=0; e<3; e++) {
		    Vec3f v1 = triangle.vert[e];
		    Vec3f v2 = triangle.vert[(e + 1) % 3];
		      
		    double t;
		    triangle2.intersectSegment(v1, v2, t);
		    Vec3f v = v1*(1.0-t) + v2*t;
		    double edist = (vert[vi] - v).len();
			
		    printf("  edge distance: %f\n", edist);

		    if (edist < edgeDistance) {
		      edgeDistance = edist;
		      edge = e;
		    }
		  }

		  added = added_points.size();

		  sep_edge_points[edge].push_back(vup);
		  vert_to_added[vup] = added_points.size();
		  added_by_this_face.push_back(added_points.size());
		  added_points.push_back(AddedPoint(vup, trii, -1, -1, false));
		  added_points.back().hasBottomPoint = true;
		
		  int other = edge_to_point[tris[trii].vi[isected_edges[0]]*stride + 
					    tris[trii].vi[(isected_edges[0] + 1)%3]];
		  added_points.back().adjacent.insert(vert_to_added[other]);
		  added_points[vert_to_added[other]].adjacent.insert(added);

		  printf(" %d: %f  %d, %d\n", k, diff, added, vup);

		  // There should only be one vertex that matches an existing point.
		  // Should check this.
		    

				    		    
		  // Two triangles: edge-io
		  int vi[3] = {tris[trii].vi[0], tris[trii].vi[1], tris[trii].vi[2]};
		  int down = triangle.above(vert[vi[isected_edges[0]]]);

		  //tris[trii].vi[0] = other;
		  //tris[trii].vi[1] = other;
		  //tris[trii].vi[2] = other;
		    
		  printf("down (edge): %d %d\n",  
			 triangle.above(vert[vi[isected_edges[0]]]),
			 triangle.above(vert[vi[(isected_edges[0] + 1) %3]]));
		  printf("%d %d %d  %d %d %d %d\n", vi[0], vi[1], vi[2], 
			 vi[isected_edges[0]], 
			 vi[(isected_edges[0] + 1)%3], vup, other);

		  tris[trii].vi[0] = vi[isected_edges[0]];
		  tris[trii].vi[1] = other + down;
		  tris[trii].vi[2] = vup + down;
		    
		  tris.push_back(Mesh::Tri());
		  tris.back().vi[0] = vup + !down;
		  tris.back().vi[1] = other + !down;
		  tris.back().vi[2] = vi[(isected_edges[0] + 1)%3];
		  break;
		}
	      }
	    }
	  }
	}
      }
	
      // There will be a sepTriAdded point for all sepTri vert that are
      // inside the tetrahedron.
      int sepTriAdded[3] = {-1, -1, -1};
      for (int k=0; k<3; k++) {    
	if (tetVertInside[triInds[k]]) {
	  printf("Vertex %d, %d/%d is inside.\n", k, triInds[k], tets->restVert.size());
	  sepTriAdded[k] = added_points.size();
	  added_points.push_back(AddedPoint(triInds[k], -1, -1, -1, false));
	}
      }

      // Need to sort the sep-tri-edge-points and create the new edges.
      for (int k=0; k<3; k++) {
	printf("sep-tri-edge: %d\n", k);

	if (sep_edge_points[k].size() == 0) continue;

	std::vector<pair<double, int> > sorted;
	  
	Vec3f v1 = triangle.vert[k];
	Vec3f v2 = triangle.vert[(k + 1) % 3];
	Vec3f dir = v2 - v1;

	for (int i=0; i<sep_edge_points[k].size(); i++) {
	  Vec3f rel = vert[sep_edge_points[k][i]] - v1;
	  double t = rel.dot(dir);
	  sorted.push_back(make_pair(t, sep_edge_points[k][i]));	    

	  printf("sorting edge %d, %f\n", i, t);
	}
	std::sort(sorted.begin(), sorted.end());

	int index = 0;
	if (sepTriAdded[k] >=0) {
	  printf("Adding from v1, %d\n", sepTriAdded[k]);
	  added_points[sepTriAdded[k]].adjacent.insert(vert_to_added[sorted[0].second]);
	  added_points[vert_to_added[sorted[0].second]].adjacent.insert(sepTriAdded[k]);	    
	  index = 1;
	}

	{
	  int i;
	  for (i = index; i+1 < sorted.size(); i+=2) {
	    int a1 = vert_to_added[sorted[i].second];
	    int a2 = vert_to_added[sorted[i+1].second];
	    printf("Adding edge %d-%d\n", a1, a2);
	    added_points[a1].adjacent.insert(a2);
	    added_points[a2].adjacent.insert(a1);
	  }
	    
	  if (i < sorted.size() && sepTriAdded[(k + 1)%3]>=0) {
	    printf("Adding to v2\n");
	    added_points[vert_to_added[sorted[index].second]].adjacent.insert(sepTriAdded[(k + 1) % 3]);
	    added_points[sepTriAdded[(k + 1) % 3]].adjacent.insert(vert_to_added[sorted[index].second]);
	  }
	}
	printf("Done sep-tri-edge.\n");
      }

      // Now there is a list of AddedPoints that can be linked together.
      // Find the chains of AddedPoints .... what about direction?
      std::map<int, int> next;
      std::vector<std::vector<int> > loops;
	

      for (int i=0; i<added_points.size(); i++) {
	printf("%d: ", i);
	added_points[i].printAdj();
      }
	
      for (int i=0; i<added_points.size(); i++) {	  
	if (!next.count(i) && added_points[i].edge) { //added_points[i].edge && !next.count(i)) {
	  int finger = i;

	  int link_length = 0;
	  int head = i;

	  bool isLoop = false;
	  vector<int> loop;

	  while (finger != -1) {	      
	    int next_finger = -1;
	    link_length++;

	    loop.push_back(finger);

	    std::set<int>::iterator it(added_points[finger].adjacent.begin());
	    std::set<int>::iterator last(added_points[finger].adjacent.end());
	      	      
	    while (it != last) {
	      if (!next.count(*it)) {
		next_finger = *it;
		next[finger] = *it;
		printf("%d => %d\n", finger, next[finger]);
		break;
	      }
	      ++it;
	    }

	    if (next_finger == -1) {
	      if (added_points[finger].adjacent.count(head)) {
		next[finger] = head;
		isLoop = true;
	      }
	      printf("tail, %d, is adjacent to head(%d): %d (nadj: %d)\n", 
		     finger, head, added_points[finger].adjacent.count(head), 
		     added_points[finger].adjacent.size());
	    }
	    finger = next_finger;
	    if (finger >=0 )
	      next[finger] = -1;
	  }
	  if (isLoop) {      
	    // FIXME: Use glu, and do all the holes at once.
	    // Triangulate the hole
	    printf("Triangulating the loop (size: %d)\n", loop.size());
	    for (int i=1; i+1<loop.size(); i++) {
	      tris.push_back(Mesh::Tri());
	      tris.back().vi[0] = added_points[loop[0]].point;
	      tris.back().vi[1] = added_points[loop[i]].point;
	      tris.back().vi[2] = added_points[loop[i+1]].point;
	    }	      

	    // Triangulate the hole (bottom)
	    for (int i=1; i+1<loop.size(); i++) {
	      tris.push_back(Mesh::Tri());
	      tris.back().vi[0] = added_points[loop[0]].bottomPoint();
	      tris.back().vi[2] = added_points[loop[i]].bottomPoint();
	      tris.back().vi[1] = added_points[loop[i+1]].bottomPoint();
	    }

	    loops.push_back(loop);
	  }	     
	  printf("link length: %d\n", link_length);
	}
      }
      printf("there were %d loops.\n", loops.size());
	
    }
  }
  initNormalsAutoSmooth(60.0); // FIXME: do this when adding new triangles.
  return isect.size();
}
