#ifndef TETRAHEDRON_H
#define TETRAHEDRON_H

#include <nmath/vec3.h>
#include <nmath/vec4.h>
#include <nmath/matrix.h>
#include <vector>

#include "TriangleIndices.h"
#include "Globals.h"

using namespace nacb;

/**
   \brief Tetrahedron, just stores indexes to a vertex list.
*/
class Tetrahedron {
 public:
  static const TriangleIndices tris[4];

  Vec3f      color;
  Vec3f normals[4];
  int vi[4];
    
  double V;
  Matrix K;
  Matrix B;
  Matrix E;

  Matrix P;
  Matrix e_plastic;

  struct Material {
    double c_yield;
    double c_max;
    double c_creep;
    double e;
    double v; 

    Material() {
      c_yield = kDefault_c_yield;
      c_max = kDefault_c_max;
      c_creep = kDefault_c_creep;

      e = 25000; //250000// Elastic modulus ???
      v = 0.1;//0.43; // Poisson's ratio
    }

  }plasticity;

  Tetrahedron();
  Tetrahedron(int v1, int v2, int v3, int v4);

  bool replaceVert(int vold, int vnew);

  bool containsVertex(int v) const;
  int indexOfVertex(int v) const;
  void updatePlasticity(const std::vector<Vec3f> & restVert,
			const std::vector<Vec3f> & vert,
			const Matrix & Rinv,
			double dt);

  void updateNormals(const std::vector<Vec3f> & vert);

  void drawSelect(const std::vector<Vec3f> & vert);
  void draw(const std::vector<Vec3f> & vert);

  bool intersectSegment(const std::vector<Vec3f> & vert, 
			const Vec3f & p1, const Vec3f & p2, 
			double & t, double eps = 1e-5) const;

  Matrix getVertexVector(const std::vector<Vec3f> & vert) const;

  Vec3f getWorldCoordinates(const std::vector<Vec3f> & vert,
			    const Vec4f & bary) const;

  double distanceToTriangle(const std::vector<Vec3f> & vert,
			    int tri, const Vec3f & pt) const;

  double distance(const std::vector<Vec3f> & vert,
		  const Vec3f & pt) const;

  Matrix getBaryMatrix(const std::vector<Vec3f> & vert) const;
  void updateVolume(const std::vector<Vec3f> & vert);
  std::ostream & print(std::ostream & o) const;
};

#endif // TETRAHEDRON
