#ifndef TRIANGLE_INDICES_H
#define TRIANGLE_INDICES_H

/**
   \brief Data structure to represent indices into a list of vertices.
   Better than 3 ints.  Useful with tetrahedra to map the 
   tet triangles to real indices.
*/
class TriangleIndices {
public:
  union {
    int data[3];
    struct {
      int v1, v2, v3;
    };
  };

  TriangleIndices(int * inds) : v1(inds[0]), v2(inds[1]), v3(inds[2]) {

  }

  TriangleIndices(int _v1 = 0, int _v2 = 0, int _v3 = 0) : v1(_v1), v2(_v2), v3(_v3) {

  }
  
  const int & operator[](int index) const {
    return data[index];
  }
  
  int & operator[](int index) {
    return data[index];
  }

  /**
     Replace the old vertex with the new one.
     \param vold  the old vertex.
     \param vnew  the new vertex index.
     \return  true if this triangle contained the vold.
  */
  bool replaceVert(int vold, int vnew){
    for (int i=0; i<3; i++) {
      if(data[i] == vold) {
	data[i] = vnew;
	return true;
      }
    }
    return false;
  }

  /**
     Map the indices through vert.
     \param vert the indices to map through
   */
  TriangleIndices mapIndices(const int vert[]) const {
    return TriangleIndices(vert[v1], vert[v2], vert[v3]);
  }

  /**
     Make the vertex v the first vertrex in the triangle.
     \param v  The vertex to shift to the first location.
     \return true if the vertex v is found.
   */
  bool makeFirst(int v) {
    for (int k=0; k<3; k++)
      if (data[k] == v) {
	TriangleIndices old(*this);
	
	for (int j=0; j<3; j++){
	  data[j] = old[(k + j) % 3];
	}
	return true;
      }
    return false;
  }

  operator const int * () {
    return data;
  }
};


#endif // TRIANGLE_INDICES_H
