/**
   An implementation of Interactive virtual materials.
*/

#include <GL/glew.h>

#include <stdio.h>
#include <ik/mesh.h>
#include <vector>
#include <set>
#include <utigl/glwindow.h>
#include <nmisc/timer.h>
#include <algorithm>

#include <boost/archive/text_oarchive.hpp>
#include <boost/archive/text_iarchive.hpp>
#include <boost/serialization/vector.hpp>
#include <boost/serialization/set.hpp>

#include <fstream>
#include <nmath/vec4.h>
#include <unistd.h>
#include <stdarg.h>

#include "Scene.h"


using namespace nacb;
using namespace std;


class Win: public GLWindow {

public:
  Scene * scene;

  Win(const char * tetName = 0, const char * tetBound = 0){
    glewInit();
    
    if (tetBound == 0) 
      scene = new Scene(tetName);
    else
      scene = new Scene(tetName, tetBound);
    
    scene->view = this;

    FFont font("fonts/Vera.ttf", 12);
    font.scaleToBe(6, height());
    UI::setFont(font);
  }
 
  void motion(int x, int y){
    if(bdown == 1){
      scene->moveHandleForView(this, x, y);
      return;
    }
    GLWindow::motion(x, y);
  }

  void drawScene() {
    scene->draw();

    if (scene->simulating) {
      scene->simulate(0.01);
#ifdef _WIN32
      Sleep(1);
#else
      usleep(1000);
#endif
      refresh();
    }
  }

  bool mouse(int button, bool down, int x, int y){
    scene->selectHandleForView(this, button, down, x, y);

    bool ret = GLWindow::mouse(button, down, x, y);
    refresh();
    return ret;
  }

  bool keyboard(unsigned char c, int x, int y){
    scene->handleKeyboard(c, x, y);
    refresh();
  }
};



int main(int ac, char * av[]){
  if(ac <= 1){
    fprintf(stderr, "Need a mesh, and optionally a mesh to bind with tets.\n");
    return 0;
  }

  Win * win = new Win(av[1], ac >= 3 ? av[2] : 0);
  win->loop(1);

  return 0;
}
