#include "Canvas.h"
#include <wx/dcclient.h>

Canvas::Canvas(wxWindow * parent, wxWindowID wid,
               const wxSize & size) : 
  wxGLCanvas(parent, wid, wxDefaultPosition, size, wxFULL_REPAINT_ON_RESIZE, wxT("gl canvas"), attribList) {
  shiftDown = 0;
  timer = new wxTimer(this);
  s_canvas = this;
  context = 0;

  mode = MOUSE_MODE;
}


void Canvas::OnKeyDown(wxKeyEvent & event){
  if (scene.get()) {
    scene->handleKeyboard(event.GetKeyCode(), 0, 0);
  }
  Refresh();
}


void Canvas::OnKeyUp(wxKeyEvent & event){
  printf("Key up.\n");
}


void Canvas::OnMouseAll(wxMouseEvent & mouseEvent){
  static int bdown;

  if (!scene) return;

  if (mouseEvent.ButtonDown())
    bdown = mouseEvent.GetButton();
  else if (mouseEvent.ButtonUp())
    bdown = 0;

  // Default is to 
  if (mouseEvent.Button(1)) {
    MySetCurrent();

    if (mode == MOUSE_MODE) {
      glViewport(0, 0, width(), height());
      scene->selectHandleForView(this, 0, mouseEvent.ButtonDown(), 
				 mouseEvent.GetX(), mouseEvent.GetY());
    }
    else {
      glViewport(0, 0, width(), height());
      scene->removeSpringForView(this, 0, mouseEvent.ButtonDown(), 
				 mouseEvent.GetX(), mouseEvent.GetY());
    }
    Refresh();
    return;
  }
  else if (mouseEvent.Dragging() && bdown == 1) {
    MySetCurrent();

    if (mode == MOUSE_MODE) {
      scene->moveHandleForView(this, mouseEvent.GetX(), mouseEvent.GetY());
    }    
    Refresh();
    return;
  }

  if (mouseEvent.ButtonDown()) {
    mouse(mouseEvent.GetButton() - 1, true, mouseEvent.GetX(), mouseEvent.GetY());
  }
  else if (mouseEvent.ButtonUp()) {
    mouse(mouseEvent.GetButton() - 1, false, mouseEvent.GetX(), mouseEvent.GetY());
  }
  else if (mouseEvent.Dragging()) {
    motion(mouseEvent.GetX(), mouseEvent.GetY());
  }

  shiftDown = mouseEvent.ShiftDown();
}

void Canvas::OnTimer(wxTimerEvent & timerEvent) {
  if (scene) {
    Refresh();
    if (scene->simulating)
      scene->simulate(0.03); 
    
#ifdef WIN32
    Sleep(1);
#else
    usleep(200);
#endif
  }
}


void Canvas::Play() {
  scene->simulating = true;
  if (!timer->IsRunning()) 
    timer->Start(30);
}


void Canvas::Stop() {
  scene->simulating = false;
  timer->Stop();
}

void Canvas::MySetCurrent() {
  /*
  if (!GetContext() && !context)
    return;
  SetCurrent(*context);
  */
  /*
  if (context)
    context->SetCurrent();
  else 
  */
  SetCurrent();
}

void Canvas::OnPaint(wxPaintEvent & ){
  wxPaintDC dc(this);
  
#ifndef __WXMOTIF__
    if (!GetContext()) return;
#endif

  MySetCurrent();

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  applyProjection();

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  applyModelview();
  
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  if (scene) {
    scene->draw();
  }  
  
  SwapBuffers();
}

Canvas * Canvas::s_canvas = 0;


int Canvas::attribList[10] = {WX_GL_RGBA,WX_GL_DOUBLEBUFFER,
			      WX_GL_STENCIL_SIZE,8, 
			      WX_GL_DEPTH_SIZE, 24, 0, 0};


BEGIN_EVENT_TABLE(Canvas, wxGLCanvas)
 EVT_PAINT(Canvas::OnPaint)
 EVT_SIZE(Canvas::OnSizeEvent)
 EVT_KEY_DOWN(Canvas::OnKeyDown)
 EVT_KEY_UP(Canvas::OnKeyUp)
 EVT_MOUSEWHEEL(Canvas::OnMouseAll)
 EVT_LEFT_UP(Canvas::OnMouseAll)
 EVT_LEFT_DOWN(Canvas::OnMouseAll)
 EVT_MIDDLE_UP(Canvas::OnMouseAll)
 EVT_MIDDLE_DOWN(Canvas::OnMouseAll)
 EVT_RIGHT_UP(Canvas::OnMouseAll)
 EVT_RIGHT_DOWN(Canvas::OnMouseAll)
 EVT_MOTION(Canvas::OnMouseAll)
 EVT_LEFT_DCLICK(Canvas::OnMouseAll)
 EVT_TIMER(-1, Canvas::OnTimer)

 EVT_ERASE_BACKGROUND(Canvas::OnEraseBackground)
//EVT_ENTER_WINDOW(Canvas::OnMouseEnterExit)
//EVT_LEAVE_WINDOW(Canvas::OnMouseEnterExit)
END_EVENT_TABLE()
