#ifndef VIRTMAT_CANVAS_H
#define VIRTMAT_CANVAS_H

#include <GL/glew.h>

#include <wx/glcanvas.h>
#include <utigl/glwindow.h>
#include <wx/timer.h>

#include "../Scene.h"

class wxTimerEvent;

class Canvas : public wxGLCanvas, public InteractiveCameraView {
public:
  static int attribList[10];

  enum {
    MOUSE_MODE,
    REMOVE_SPRING_MODE
  }mode;

  Canvas(wxWindow * parent = 0, wxWindowID wid = wxID_ANY, 
	 const wxSize & size = wxSize(600, 400));

  ~Canvas() { 
    if (timer) {
      timer->Stop();
      delete timer;
      timer = 0;
    }
  }

  void OnSizeEvent(wxSizeEvent & size){
    wxGLCanvas::OnSize(size);
    printf("Canvas resizing: %d %d\n", 
	   size.GetSize().GetWidth(),
	   size.GetSize().GetHeight());
    SetCurrent();
    glViewport(0, 0, size.GetSize().GetWidth(), size.GetSize().GetHeight());
    Refresh();
  }

  void MySetCurrent();

  void OnKeyDown(wxKeyEvent & event);
  void OnKeyUp(wxKeyEvent & event);
  void OnMouseAll(wxMouseEvent & mouse);  
  void OnPaint(wxPaintEvent & paint);
  void OnTimer(wxTimerEvent & timerEvent);
  void OnEraseBackground(wxEraseEvent &){ }
  void SetRemoveSpringMode() { 
    mode = REMOVE_SPRING_MODE;
    SetCursor(wxCursor(wxCURSOR_CROSS));
  }
  void SetMouseMode() {
    mode = MOUSE_MODE;
    SetCursor(wxCursor(wxCURSOR_ARROW));
  }
  void Play();
  void Stop();

  void SetContext(wxGLContext * con) {
    context = con;
  }

  int width() {
    int w, h;
    GetSize(&w, &h);
    return w;
  }

  int height() {
    int w, h;
    GetSize(&w, &h);
    return h;
  }

  bool shiftActive() {
    return shiftDown;
  }

  void refresh() {
    Refresh();
  }

  static Canvas * instance() {
    return s_canvas;
  }

  wxGLContext * context;

  wxTimer * timer;

 public:
  static Canvas * s_canvas;

  Scene::ptr scene;  
  bool shiftDown;

  DECLARE_EVENT_TABLE();
};

#endif

