#include "Options.h"
#include "Canvas.h" // For global access to scene

#include "../Scene.h"
#include "../TetMesh.h"


TetMesh::ptr & getTetMesh() {
  static TetMesh::ptr ret;
  if (!Canvas::instance()) return ret;
  Scene::ptr scene = Canvas::instance()->scene;
  if (scene)
    return scene->tetmesh;
  return ret;
}

Options::Options(wxWindow * parent) : wxDialog(parent, OPTIONS_DIALOG_ID, wxT("Options"), 
					       wxDefaultPosition, wxSize(220, 300)){
  wxStaticBoxSizer * options = new wxStaticBoxSizer(wxVERTICAL, this, wxT("Material"));
  
  wxSizer * hbox = new wxBoxSizer(wxHORIZONTAL);
  wxString presets[4] = {wxT("Soft"), wxT("Medium"), 
			 wxT("Stiff"), wxT("Real Stiff")};
  presetsChoice = new wxChoice(this, PRESET_CHOICE_ID, 
			       wxDefaultPosition, wxDefaultSize, 4, presets);
  presetsChoice->SetSelection(1);
  hbox->Add(new wxStaticText(this, -1, wxT("Presets")), 1, wxEXPAND | wxRIGHT);
  hbox->Add(presetsChoice);
  options->Add(hbox, 1, wxEXPAND | wxGROW);

  hbox = new wxBoxSizer(wxHORIZONTAL);
  hbox->Add(new wxStaticText(this, -1, wxT("Poisson Ratio")), 1, wxEXPAND | wxRIGHT);
  poissonSlider = new wxSlider(this, POISSON_SLIDER_ID, 10, 0, kSliderMax);
  poissonSlider->SetSize(12, 100);
  hbox->Add(poissonSlider, 1, wxEXPAND | wxGROW);
  options->Add(hbox, 1, wxEXPAND | wxGROW);

  hbox = new wxBoxSizer(wxHORIZONTAL);
  hbox->Add(new wxStaticText(this, -1, wxT("Elastic Modulus")), 1, wxEXPAND | wxRIGHT);
  elasticModSlider = new wxSlider(this, ELASTIC_MOD_SLIDER_ID, 10, 0, kSliderMax);
  hbox->Add(elasticModSlider, 1, wxEXPAND | wxGROW);
  options->Add(hbox, 1, wxEXPAND | wxGROW);

  wxStaticBoxSizer * plasticOptions = new wxStaticBoxSizer(wxVERTICAL, this, wxT("Plasticity"));
  hbox = new wxBoxSizer(wxHORIZONTAL);
  hbox->Add(new wxStaticText(this, -1, wxT("Yield")), 1, wxEXPAND | wxRIGHT);    
  plasticYieldSlider = new wxSlider(this, PLASTIC_YIELD_SLIDER_ID, 10, 0, kSliderMax);
  hbox->Add(plasticYieldSlider, 1, wxEXPAND | wxGROW);
  plasticOptions->Add(hbox, 1, wxEXPAND | wxGROW);
    
  hbox = new wxBoxSizer(wxHORIZONTAL);
  hbox->Add(new wxStaticText(this, -1, wxT("Creep")), 1, wxEXPAND | wxRIGHT);    
  plasticCreepSlider = new wxSlider(this, PLASTIC_CREEP_SLIDER_ID, 10, 0, kSliderMax);
  hbox->Add(plasticCreepSlider, 1, wxEXPAND | wxGROW);
  plasticOptions->Add(hbox, 1, wxEXPAND | wxGROW);


  hbox = new wxBoxSizer(wxHORIZONTAL);
  hbox->Add(new wxStaticText(this, -1, wxT("Max")), 1, wxEXPAND | wxRIGHT);    
  plasticOptions->Add(hbox, 1, wxEXPAND | wxGROW);
  plasticMaxSlider = new wxSlider(this, PLASTIC_MAX_SLIDER_ID, 10, 0, kSliderMax);
  hbox->Add(plasticMaxSlider, 1, wxEXPAND | wxGROW);
  options->Add(plasticOptions, 3, wxEXPAND | wxGROW);

  SetSizer(options);

  // Set the bounds for the sliders (linear mapping)
  sliderMapping[poissonSlider] = SliderMapping::ptr(new SliderMapping(0.05, 0.49));
  sliderMapping[elasticModSlider] = SliderMapping::ptr(new SliderMapping(4000, 1000000));
  sliderMapping[plasticYieldSlider] = SliderMapping::ptr(new SliderMapping(0, 1));
  sliderMapping[plasticCreepSlider] = SliderMapping::ptr(new SliderMapping(0, 100));
  sliderMapping[plasticMaxSlider] = SliderMapping::ptr(new SliderMapping(0, 1000));

  transferFrom(getTetMesh());
}


void Options::OnPoissonChanged(wxScrollEvent & event) {
  transferTo(getTetMesh());
}

void Options::OnElasticModChanged(wxScrollEvent & event) {
  transferTo(getTetMesh());
}

void Options::OnPlasticCreepChanged(wxScrollEvent & event) {
  transferTo(getTetMesh());
}

void Options::OnPlasticYieldChanged(wxScrollEvent & event) {
  transferTo(getTetMesh());
}


void Options::OnPlasticMaxChanged(wxScrollEvent & event) {
  transferTo(getTetMesh());
}

void Options::OnPreset(wxCommandEvent & event) {
  double emods[4] = {15000, 20000, 250000, 1000000};
  poissonSlider->SetValue(sliderMapping[poissonSlider]->toSliderValue(0.4));
  elasticModSlider->SetValue(sliderMapping[elasticModSlider]->toSliderValue(emods[event.GetSelection()]));
  transferTo(getTetMesh());
}

void Options::transferTo(TetMesh::ptr & tetmesh) {
  if (!tetmesh.get()) return;
  
  Tetrahedron::Material material = tetmesh->getMaterial();
  material.e = sliderMapping[elasticModSlider]->fromSliderValue(elasticModSlider->GetValue());
  material.v = sliderMapping[poissonSlider]->fromSliderValue(poissonSlider->GetValue());
  material.c_creep = sliderMapping[plasticCreepSlider]->fromSliderValue(plasticCreepSlider->GetValue());
  material.c_yield = sliderMapping[plasticYieldSlider]->fromSliderValue(plasticYieldSlider->GetValue());
  material.c_max = sliderMapping[plasticMaxSlider]->fromSliderValue(plasticMaxSlider->GetValue());
  
  tetmesh->setMaterial(material);
}

void Options::transferFrom(TetMesh::ptr & tetmesh) {
  if (!tetmesh.get()) return;

  Tetrahedron::Material material = tetmesh->getMaterial();
  elasticModSlider->SetValue(sliderMapping[elasticModSlider]->toSliderValue(material.e));  
  poissonSlider->SetValue(sliderMapping[poissonSlider]->toSliderValue(material.v)); 
  plasticYieldSlider->SetValue(sliderMapping[plasticYieldSlider]->toSliderValue(material.c_yield));  
  plasticCreepSlider->SetValue(sliderMapping[plasticCreepSlider]->toSliderValue(material.c_creep)); 
  plasticMaxSlider->SetValue(sliderMapping[plasticMaxSlider]->toSliderValue(material.c_max)); 
}


BEGIN_EVENT_TABLE(Options, wxDialog)
 EVT_CHOICE(PRESET_CHOICE_ID, Options::OnPreset)
 EVT_COMMAND_SCROLL(POISSON_SLIDER_ID, Options::OnPoissonChanged)
 EVT_COMMAND_SCROLL(ELASTIC_MOD_SLIDER_ID, Options::OnElasticModChanged)
 EVT_COMMAND_SCROLL(PLASTIC_YIELD_SLIDER_ID, Options::OnPlasticYieldChanged)
 EVT_COMMAND_SCROLL(PLASTIC_CREEP_SLIDER_ID, Options::OnPlasticCreepChanged)
 EVT_COMMAND_SCROLL(PLASTIC_MAX_SLIDER_ID, Options::OnPlasticMaxChanged)

END_EVENT_TABLE()
