#include <wx/frame.h>
#include <wx/statbox.h>
#include <wx/stattext.h>
#include <wx/sizer.h>
#include <wx/dialog.h>
#include <wx/choice.h>
#include <wx/slider.h>
#include <map>
#include <boost/shared_ptr.hpp>

#include "../TetMesh.h"

enum {
  OPTIONS_DIALOG_ID = wxID_HIGHEST + 100,
  PRESET_CHOICE_ID,
  POISSON_SLIDER_ID,
  ELASTIC_MOD_SLIDER_ID,
  PLASTIC_YIELD_SLIDER_ID,
  PLASTIC_CREEP_SLIDER_ID,
  PLASTIC_MAX_SLIDER_ID,
};


const static int kSliderMax = 1000000;

/**
   Class to conver the mapping from the integral slider values
   to the double values of the parameters.
 */
class SliderMapping {
 public:
  SliderMapping(double n, double x) {
    minValue = n;
    maxValue = x;
  }
  
  double fromSliderValue(int v) {
    // Just do a simple linear mapping.
    return double(v)/kSliderMax*(maxValue - minValue) + minValue;
  }

  int toSliderValue(double v) {
    return (int) round((v - minValue)/(maxValue - minValue) * kSliderMax);
  }
  
 protected:
  double minValue, maxValue;

 public:
  typedef boost::shared_ptr<SliderMapping> ptr;
};




class Options: public wxDialog {
 public:
  Options(wxWindow * parent);

  void transferTo(TetMesh::ptr & tetMesh);
  void transferFrom(TetMesh::ptr & tetMesh);

  void OnPoissonChanged(wxScrollEvent & event);
  void OnElasticModChanged(wxScrollEvent & event);
  void OnPlasticCreepChanged(wxScrollEvent & event);
  void OnPlasticYieldChanged(wxScrollEvent & event);
  void OnPlasticMaxChanged(wxScrollEvent & event);
  void OnPreset(wxCommandEvent & event);

 protected:
  wxChoice * presetsChoice;
  wxSlider * poissonSlider;
  wxSlider * elasticModSlider;
  wxSlider * plasticCreepSlider;
  wxSlider * plasticYieldSlider;
  wxSlider * plasticMaxSlider;

  std::map<wxSlider *, SliderMapping::ptr> sliderMapping;

  DECLARE_EVENT_TABLE();
};


