#include "VirtMatFrame.h"
#include <wx/filedlg.h>

VirtMatFrame * VirtMatFrame::s_instance = 0;


void VirtMatFrame::Initialize() {
  printf("init.\n");
  options =  0;//new Options(this);  
  canvas = 0;
  /*
  wxBoxSizer * mainSizer = new wxBoxSizer(wxVERTICAL);
  mainSizer->Add(canvas, 1, wxGROW | wxEXPAND);
  SetSizer(mainSizer);
  */
  wxMenu  * menus[3];
  wxString titles[3] = {wxT("&File"), wxT("&Edit"), wxT("&View")};
   
  menus[0] = new wxMenu(wxT(""));

  menus[0]->Append(wxID_OPEN, wxT("&Open"), wxT("Open a file."));
  menus[0]->AppendSeparator();
  menus[0]->Append(wxID_EXIT, wxT("&Exit"), wxT("Exit."));

  menus[1] = new wxMenu(wxT(""));
  menus[1]->AppendCheckItem(TOOL_PLAY_ID, wxT("Play\tCtrl+P"));
  menus[1]->Append(TOOL_RESET_ID, wxT("&Reset"), 
		   wxT("Reset the state of the simulation"));
  menus[1]->AppendSeparator();

  menus[1]->AppendRadioItem(TOOL_MOUSE_SPRING_ID, wxT("Move\tCtrl+M"));
  menus[1]->AppendRadioItem(TOOL_REMOVE_SPRING_ID, wxT("Remove Spring\tDelete"));
  menus[1]->Append(TOOL_ADD_SPRING_ID, wxT("Add Spring\tCtrl+C"));

  menus[1]->AppendSeparator();
  menus[1]->AppendCheckItem(TOOL_DRAW_TETS_ID, wxT("&Show tets"), 
			    wxT("Show the tetrahedra"));
  menus[1]->AppendCheckItem(TOOL_WIREFRAME_ID, wxT("&Wireframe"), 
			    wxT("Show the tetrahedra"));

  menus[2] = new wxMenu(wxT(""));
  menus[2]->Append(SHOW_OPTIONS_ID, wxT("Options"), 
		   wxT("Popup options dialog"));

  wxMenuBar * menuBar = new wxMenuBar(3, menus, titles);
  SetMenuBar(menuBar);

  printf("Creating canvas\n");
  canvas = new Canvas(this);

  /*
  wxToolBar * toolBar = CreateToolBar();
  toolBar->AddCheckTool(TOOL_PLAY_ID, wxT("Play"),
 			wxBitmap(_T("gfx/play.gif")), 
			wxBitmap(_T("gfx/play.gif")), _T("Play"));
  //toolBar->AddTool(TOOL_RESET_ID, wxT("Reset"));
  toolBar->AddSeparator();
  SetToolBar(toolBar);
  */
  //wxGLContext * context = new wxGLContext(canvas, 0);
  //canvas->SetContext(context);

  SetFocus();
  if (canvas) 
    canvas->SetFocus();
}


void VirtMatFrame::OnMouseSpring(wxCommandEvent& event){
  canvas->SetMouseMode();
}

void VirtMatFrame::OnRemoveSpring(wxCommandEvent& event){
  canvas->SetRemoveSpringMode();
}

void VirtMatFrame::OnShowOptions(wxCommandEvent& event){
  if (!options)
    options = new Options(this);  
  options->Raise();
  options->Show(true);
}

void VirtMatFrame::OnAddSpring(wxCommandEvent& event){
  scene->convertHandleToSpring();
  canvas->Refresh();
}

void VirtMatFrame::OnPlay(wxCommandEvent& event){
  if (event.IsChecked())
    canvas->Play();
  else
    canvas->Stop();
} 

void VirtMatFrame::OnDrawTets(wxCommandEvent& event){
  if (scene)
    scene->drawTets = event.IsChecked();
}

void VirtMatFrame::OnWireframe(wxCommandEvent& event) {
  if (scene)
    scene->wireframe = event.IsChecked();
}

void VirtMatFrame::OnReset(wxCommandEvent& event) {
  if (scene) 
    scene->reset();
}

void VirtMatFrame::OnOpen(wxCommandEvent&){
  wxFileDialog dialog(this,
                      wxT("Open a scene"), wxT(""),
                      wxT("scene.ivm"),
                      wxT("Scene files (*.ivm);*.ivm"), wxFD_OPEN);
  if (dialog.ShowModal() == wxID_OK) {
    wxString str = dialog.GetPath();
    
    std::string name = (const char*)str.mb_str(wxConvUTF8);
    printf("Loading %s\n", name.c_str());
    Load(name.c_str());
    Refresh();
  }
} 

void VirtMatFrame::OnExit(wxCommandEvent&) {
  Close();
}

void VirtMatFrame::Load(const char * path) {
  if (path) {
    canvas->Show(true);
    
    FFont font("fonts/Vera.ttf", 12);
    font.scaleToBe(6, 640);
    UI::setFont(font);

    printf("Loading the scene, %s\n", path);
    scene = Scene::ptr(new Scene(path));
    
    printf("Done loading the scene.\n");
    canvas->scene = scene;

    printf("Setting the view.\n");
    scene->view = dynamic_cast<InteractiveCameraView *>(canvas);

    if (options) 
      options->transferFrom(scene->tetmesh);
  }
}

void VirtMatFrame::OnSizeEvent(wxSizeEvent& sizeEvent) {
  wxFrame::OnSize(sizeEvent);
  printf("Size.\n");
  if (canvas)
    canvas->Refresh();
  Refresh();
}

BEGIN_EVENT_TABLE(VirtMatFrame, wxFrame)
 EVT_SIZE(VirtMatFrame::OnSizeEvent)
 EVT_MENU(wxID_EXIT, VirtMatFrame::OnExit)
 EVT_MENU(wxID_OPEN, VirtMatFrame::OnOpen)
 EVT_MENU(SHOW_OPTIONS_ID, VirtMatFrame::OnShowOptions)
 EVT_TOOL(TOOL_PLAY_ID, VirtMatFrame::OnPlay)
 EVT_TOOL(TOOL_RESET_ID, VirtMatFrame::OnReset)

 EVT_TOOL(TOOL_MOUSE_SPRING_ID, VirtMatFrame::OnMouseSpring)
 EVT_TOOL(TOOL_REMOVE_SPRING_ID, VirtMatFrame::OnRemoveSpring)
 EVT_TOOL(TOOL_ADD_SPRING_ID, VirtMatFrame::OnAddSpring)


 EVT_MENU(TOOL_DRAW_TETS_ID,  VirtMatFrame::OnDrawTets)
 EVT_MENU(TOOL_WIREFRAME_ID, VirtMatFrame::OnWireframe)

 EVT_SIZE(VirtMatFrame::OnSize)
END_EVENT_TABLE()
