#ifndef VIRT_MAT_FRAME_H
#define VIRT_MAT_FRAME_H

#include <GL/glew.h>

#include <wx/app.h>
#include <wx/menu.h>
#include <wx/glcanvas.h>
#include <wx/sizer.h>
#include <wx/toolbar.h>
#include <wx/choice.h>
#include <nimage/image.h>
#include <wx/frame.h>
#include <GL/glut.h>
#include <nmath/quaternion.h>
#include <nmath/vec3.h>
#include <wx/slider.h>
#include "Canvas.h"
#include "Options.h"

#include "../Scene.h"

enum {
  MAIN_WINDOW_ID = wxID_HIGHEST,
  
  TOOL_PLAY_ID,
  TOOL_RESET_ID,
  TOOL_MOUSE_SPRING_ID,
  TOOL_ADD_SPRING_ID,
  TOOL_REMOVE_SPRING_ID,
  SHOW_OPTIONS_ID,

  TOOL_DRAW_TETS_ID,
  TOOL_WIREFRAME_ID
};


/**
   The main window, holds access to the global scene.
*/
class VirtMatFrame: public wxFrame {
public:
  VirtMatFrame() : wxFrame(0, MAIN_WINDOW_ID, wxT("Kin"), 
			   wxDefaultPosition, wxSize(640, 480)) {
    canvas = 0;
    Initialize();
    s_instance = this;
  }

  void Initialize();
  void OnDrawTets(wxCommandEvent&);
  void OnWireframe(wxCommandEvent&);
  void OnShowOptions(wxCommandEvent&);
  void OnOpen(wxCommandEvent&);
  void OnExit(wxCommandEvent&);
  void OnPlay(wxCommandEvent&);
  void OnReset(wxCommandEvent&);
  void OnAddSpring(wxCommandEvent& event);
  void OnRemoveSpring(wxCommandEvent& event);
  void OnMouseSpring(wxCommandEvent& event);
  
  void Load(const char * path);
  void OnSizeEvent(wxSizeEvent&);
  Scene::ptr & getScene() {
    assert(s_instance);
    return s_instance->scene;
  }

protected:
  static VirtMatFrame * s_instance;
  
public:
  
  Scene::ptr   scene;
  Canvas    * canvas;
  Options  * options;
  DECLARE_EVENT_TABLE();
};


#endif 
